# -*- coding: utf-8 -*-
from typing import Optional, List
from ..client.mongo_client import Mongo
from ..model.entity import StreamSpider
from ..model.enums import CommonStatus


# 获取portainer爬虫配置
def get_stream_spiders(spider_ids: List[str], status: CommonStatus) -> Optional[StreamSpider]:
    row = Mongo().get('zcbot_stream_spider', {'spiderId': {'$in': spider_ids}, 'status': status.name})
    if not row:
        return None
    return StreamSpider(**row)


def get_stream_spider(spider_id: str, status: CommonStatus) -> Optional[StreamSpider]:
    row = Mongo().get('zcbot_stream_spider', {'spiderId': spider_id, 'status': status.name})
    if not row:
        return None
    return StreamSpider(**row)


# 获取支持网站平台
def get_platforms_by_spider_id(spider_ids: List[str], status: CommonStatus):
    return Mongo().aggregate('zcbot_platforms', [
        {
            '$lookup': {
                'from': 'zcbot_stream_spider',
                'localField': '_id',
                'foreignField': 'plat_code',
                'as': 'spider'
            }
        },
        {
            '$match': {'spider._id': {'$in': spider_ids}, 'status': status.name}
        },
        {
            '$project': {'spider': 0}
        },
        {'$sort': {'sort': 1}}
    ])



from typing import List

from pydantic import BaseModel, Field

from ..util import time as time_lib


class BaseData(BaseModel):
    """
    通用基础数据模型
    """
    # mongodb主键
    _id: str = None
    # 插入时间
    genTime: int = Field(
        default_factory=time_lib.current_timestamp10
    )


class Platforms(BaseModel):
    """
    支持平台模型
    """
    # 平台编码
    platCode: str = None
    # 平台名称
    platName: str = None
    # 排序
    sort: int = None
    # logo
    logo: str = None


class Rule(BaseModel):
    """
    解析规则
    """
    # host
    host: str = None
    # 平台编码
    platCode: str = None
    # 参数
    params: List = []
    # 解析规则
    regex: List = []
    # 排序
    sort: int = None
    # 生成时间
    genTime: int = None



# -*- coding: utf-8 -*-
import json
import requests
from ..util import logger
from ..util.decator import singleton
from ..model.entity import PortainerNode, BatchSpider

LOGGER = logger.get('容器')


@singleton
class PortainerClient(object):
    """
    Portainer客户端简易封装
    """

    def __init__(self):
        self.user_agent = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.25 Safari/537.36 Core/1.70.3775.400 QQBrowser/10.6.4208.400'
        # 客户端
        self.create_api = '{}/api/endpoints/{}/docker/containers/create?name={}'
        self.start_api = '{}/api/endpoints/{}/docker/containers/{}/start'
        self.stop_api = '{}/api/endpoints/{}/docker/containers/{}/kill'

    def create_container(self, node: PortainerNode, spider: BatchSpider, container_name: str, param_data):
        container_conf = {
            "Cmd": [
                "/bin/sh",
                "-c",
                spider.param % param_data
            ],
            "Env": spider.env,
            "Image": spider.dockerImage
        }

        rs = requests.post(
            url=self.create_api.format(node.apiBaseUrl, node.endpointId, container_name),
            headers={
                'X-API-Key': node.apiToken,
                'User-Agent': self.user_agent,
                'Content-Type': 'application/json;charset=UTF-8',
            },
            data=json.dumps(container_conf)
        )
        if rs:
            data = json.loads(rs.text) or {}
            if data.get('Id', None):
                id = data.get('Id', '')
                LOGGER.info(f'创建成功: id={id}, endpoint={node.endpointId} name={container_name}')
                return id

        LOGGER.error(f'创建失败: result={rs.text}, endpoint={node.endpointId} name={container_name}')

    def start(self, node: PortainerNode, container_id: str):
        rs = requests.post(
            url=self.start_api.format(node.apiBaseUrl, node.endpointId, container_id),
            headers={
                'X-API-Key': node.apiToken,
                'User-Agent': self.user_agent,
                'Content-Type': 'application/json;charset=UTF-8',
            }
        )
        if rs and rs.status_code in [204, 200]:
            LOGGER.info(f'启动成功: id={container_id}')
            return container_id

        LOGGER.error(f'启动失败: id={container_id}')

    def stop(self, node: PortainerNode, container_id: str):
        rs = requests.post(
            url=self.stop_api.format(node.apiBaseUrl, node.endpointId, container_id),
            headers={
                'X-API-Key': node.apiToken,
                'User-Agent': self.user_agent,
                'Content-Type': 'application/json;charset=UTF-8',
            }
        )
        if rs and rs.status_code in [204, 200]:
            LOGGER.info(f'停止成功: id={container_id}')
            return container_id

        LOGGER.error(f'停止失败: id={container_id}')



# -*- coding: utf-8 -*-
from typing import Optional, List
from ..client.mongo_client import Mongo
from ..model.entity import PortainerNode
from ..model.enums import CommonStatus


# 获取节点信息
def get_node(node_id) -> Optional[PortainerNode]:
    rs = Mongo().get(collection='zcbot_portainer_node', query={'_id': node_id})
    if not rs:
        return None

    return PortainerNode(**rs)


# 获取节点信息列表
def get_node_list(query: dict = None, fields: dict = None, include_disable: bool = False) -> List[PortainerNode]:
    _query = query or {}
    _fields = fields or {}
    if not include_disable:
        _query['status'] = CommonStatus.ON.name
    rows = Mongo().list(collection='zcbot_portainer_node', query=_query, fields=_fields)
    if rows:
        return [PortainerNode(**row) for row in rows]

    return []



# -*- coding: UTF-8 -*-


# rabbitmq结果集交换机名称
def get_rabbit_result_exchange_key():
    return f'ex.zcbot_result'


# rabbitmq路由名称
def get_rabbit_result_routing_key(batch_id):
    return f'rt.result_{batch_id}'


# rabbitmq结果集队列名称
def get_rabbit_result_queue_key(batch_id):
    return f'qu.result_{batch_id}'


# 测试
if __name__ == '__main__':
    # print(get_rabbit_result_queue_key(210100110011))
    pass
